package org.orbstudio.wizards.client;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.util.Properties;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.orbstudio.ORBStudioPlugin;
import org.orbstudio.common.AppKeys;
import org.orbstudio.common.SystemUtil;
import org.orbstudio.exception.IDLFileNotFoundException;
import org.orbstudio.exception.ProjectNotOpenedException;
import org.orbstudio.popup.actions.factory.IORB;
import org.orbstudio.wizards.server.WizardUtil;

public class ClientImplWizard extends Wizard implements INewWizard {
	protected IStructuredSelection selection;
	protected IWorkbench workbench;
	protected String separator;

	/**
	 * @clientCardinality 1
	 * @supplierCardinality 0..1
	 */
	private ClientWizardPage clientWizardPage;

	/**
	 * ClientImplWizard
	 */
	public ClientImplWizard() {
		super();

		Properties props = System.getProperties();
		separator = props.getProperty("line.separator");
	}

	/**
	 * performFinish
	 */
	public boolean performFinish() {
		final IPackageFragmentRoot packageFragmentRoot = clientWizardPage.getPackageFragmentRoot();
		final String packageName = clientWizardPage.getPackageName();
		final String interfaceName = clientWizardPage.getInterface();
		final String implfileName = clientWizardPage.getImplFileName();

		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
				try {
					doFinish(packageFragmentRoot, packageName, interfaceName, implfileName, monitor);
				}
				catch (CoreException ex) {
					throw new InvocationTargetException(ex);
				}
				finally {
					monitor.done();
				}
			}
		};

		try {
			this.getContainer().run(true, false, op);
		}
		catch (InvocationTargetException ex) {
			Throwable realException = ex.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException.getMessage());
			return false;
		}
		catch (InterruptedException ex) {
			return false;
		}
		return true;
	}

	/**
	 * doFinish
	 * 
	 * @param packageFragmentRoot
	 * @param packageName
	 * @param interfaceName
	 * @param implfileName
	 * @param monitor
	 * @throws CoreException
	 */
	private void doFinish(IPackageFragmentRoot packageFragmentRoot, String packageName, String interfaceName,
	    String implfileName, IProgressMonitor monitor) throws CoreException {
		IPackageFragment packageFragment = null;
		ICompilationUnit compUnitClientImplFile = null;
		IORB currentORB = ORBStudioPlugin.getDefault().getCurrnetORB();

		monitor.beginTask("Creating CORBA client implementation...", 3);

		try {
			/*
			 * Create a package directory, here.
			 */
			monitor.setTaskName("Creating the package directory...");
			packageFragment = packageFragmentRoot.createPackageFragment(packageName, false, new NullProgressMonitor());
			monitor.worked(1);

			/*
			 * Create a Java content, here.
			 */
			StringBuffer sb = null;
			try {
				sb = ORBStudioPlugin.getDefault().getClientImplFileTemplate();
			}
			catch (IOException ex) {
				WizardUtil.getInstance().throwCoreException(ex.getMessage());
			}
			String code = sb.toString();

			/*
			 * Build Java code by replacing some parameters.
			 */
			code = SystemUtil.strReplace(code, "%%%CLIENTIMPLCLASSNAME%%%", implfileName);
			code = SystemUtil.strReplace(code, "%%%CLIENTINTERFACE%%%", interfaceName);
			code = SystemUtil.strReplace(code, "%%%CLIENTPROPERTIES%%%", currentORB.getORBPropertiesCode().toString());

			/*
			 * Note. Java IDL seems not to support 'Persistence policy' so make policy
			 * parts commented out.
			 */
			if (currentORB.getORBname().equals(AppKeys.ORB_JAVAIDL)) {
				code = SystemUtil.strReplace(code, "%%%JAVAIDL_COMMENTOUT%%%", "// ");
			}
			else {
				code = SystemUtil.strReplace(code, "%%%JAVAIDL_COMMENTOUT%%%", "");
			}

			monitor.setTaskName("Creating Java implementation...");
			compUnitClientImplFile = packageFragment.createCompilationUnit(implfileName + ".java", code, true,
			  new NullProgressMonitor());
			/*
			 * Add package declare.
			 */
			if (!packageFragment.isDefaultPackage()) {
				compUnitClientImplFile.createPackageDeclaration(packageName, new NullProgressMonitor());
			}
		}
		catch (JavaModelException e) {
			WizardUtil.getInstance().throwCoreException(e.getMessage());
		}
		monitor.worked(1);

		final IFile file = (IFile) compUnitClientImplFile.getResource();

		monitor.setTaskName("Opening file for editing...");
		getShell().getDisplay().asyncExec(new Runnable() {
			public void run() {
				IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
				try {
					IDE.openEditor(page, file, true);
				}
				catch (PartInitException e) {
				}
			}
		});
		monitor.worked(1);
	}

	/**
	 * addPages
	 */
	public void addPages() {
		super.addPages();

		try {
			clientWizardPage = new ClientWizardPage(workbench, selection);
		}
		catch (IDLFileNotFoundException ex) {
			MessageDialog.openWarning(this.getShell(), "ORB Wizard", ex.getMessage());
			return;
		}
		catch (ProjectNotOpenedException ex) {
			MessageDialog.openWarning(this.getShell(), "ORB Wizard", ex.getMessage());
			return;
		}
		catch (CoreException ex) {
			MessageDialog.openError(this.getShell(), "ORB Wizard", ex.getMessage());
			return;
		}
		clientWizardPage.setTitle("CORBA Client Wizard");
		clientWizardPage.setDescription("Create a Java client implementation from the IDL file.");
		URL url = ORBStudioPlugin.getDefault().getIconImage("/icons/client_wiz.gif");
		clientWizardPage.setImageDescriptor(ImageDescriptor.createFromURL(url));
		addPage(clientWizardPage);
	}

	/**
	 * init
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.workbench = workbench;
		this.selection = selection;

		setNeedsProgressMonitor(true);
		setWindowTitle("ORBStudio CORBA Wizard");
	}
}
