package org.orbstudio.wizards.idl;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.util.Properties;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.orbstudio.ORBStudioPlugin;
import org.orbstudio.exception.ProjectNotOpenedException;
import org.orbstudio.wizards.server.WizardUtil;

public class IDLWizardAdapter extends Wizard implements INewWizard {
	protected IStructuredSelection selection;
	protected IWorkbench workbench;
	protected String separator;
	protected String title;
	protected String desc;
	protected String idlFileName;

	/**
	 * @clientCardinality 1
	 * @supplierCardinality 0..1
	 */
	protected IDLWizardPage idlWizardPage;

	/**
	 * openContentStream
	 * 
	 * @return
	 */
	public InputStream openContentStream() {
		return null;
	}

	/**
	 * IDLWizardAdapter
	 */
	public IDLWizardAdapter() {
		super();

		Properties props = System.getProperties();
		separator = props.getProperty("line.separator");
	}

	/**
	 * The worker method. It will find the container, create the file if missing
	 * or just replace its contents, and open the editor on the newly created
	 * file.
	 */
	private void doFinish(String containerName, String fileName, IProgressMonitor monitor) throws CoreException {

		// create a sample file
		monitor.beginTask("Creating " + fileName, 2);
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IResource resource = root.findMember(new Path(containerName));
		if (!resource.exists() || !(resource instanceof IContainer)) {
			WizardUtil.getInstance().throwCoreException("Container \"" + containerName + "\" does not exist.");
		}

		IContainer container = (IContainer) resource;
		final IFile file = container.getFile(new Path(fileName));
		try {
			InputStream stream = openContentStream();
			if (file.exists()) {
				file.setContents(stream, true, true, monitor);
			}
			else {
				file.create(stream, true, monitor);
			}
			stream.close();
		}
		catch (IOException ex) {
			WizardUtil.getInstance().throwCoreException("IDLWizardAdaptor#doFinish(): " + ex.getMessage());
		}
		monitor.worked(1);

		monitor.setTaskName("Opening file for editing...");

		getShell().getDisplay().asyncExec(new Runnable() {
			public void run() {
				IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
				try {
					IDE.openEditor(page, file, true);
				}
				catch (PartInitException e) {
				}
			}
		});
		monitor.worked(1);
	}

	/**
	 * This method is called when 'Finish' button is pressed in the wizard. We
	 * will create an operation and run it using wizard as execution context.
	 */
	public boolean performFinish() {
		final String containerName = idlWizardPage.getContainerName();
		final String fileName = idlWizardPage.getIDLFileName();

		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				try {
					doFinish(containerName, fileName, monitor);
				}
				catch (CoreException e) {
					throw new InvocationTargetException(e);
				}
				finally {
					monitor.done();
				}
			}
		};
		try {
			getContainer().run(true, false, op);
		}
		catch (InterruptedException e) {
			return false;
		}
		catch (InvocationTargetException e) {
			Throwable realException = e.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException.getMessage());
			return false;
		}
		return true;
	}

	/**
	 * addPages
	 */
	public void addPages() {
		super.addPages();

		try {
			idlWizardPage = new IDLWizardPage(selection);
		}
		catch (ProjectNotOpenedException ex) {
			MessageDialog.openWarning(this.getShell(), "ORB Wizard", ex.getMessage());
			return;
		}
		idlWizardPage.setTitle(title);
		idlWizardPage.setDescription(desc);
		idlWizardPage.setIDLFileName(idlFileName);

		URL url = ORBStudioPlugin.getDefault().getIconImage("/icons/idl_wiz.gif");
		idlWizardPage.setImageDescriptor(ImageDescriptor.createFromURL(url));

		addPage(idlWizardPage);
	}

	/**
	 * init
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.workbench = workbench;
		this.selection = selection;
		setNeedsProgressMonitor(true);
	}
}
