package org.orbstudio.wizards.idl;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;
import org.orbstudio.exception.ProjectNotOpenedException;

/**
 * @stereotype IDL Wizard page
 */
public class IDLWizardPage extends WizardPage {
	private Text containerText;
	private Text fileText;
	private ISelection selection;
	private String idlFileName;

	/**
	 * IDLWizardPage
	 * 
	 * @param pageName
	 * @throws ProjectNotOpenedException
	 */
	public IDLWizardPage(ISelection selection) throws ProjectNotOpenedException {
		super("wizardPage");

		this.selection = selection;
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IContainer container = getProjectContainer();
		if (container == null) {
			String msg = "No Project found.";
			throw new ProjectNotOpenedException(msg);
		}
		IProject project = root.getProject(container.getName());

		/*
		 * Check if the project already opened.
		 */
		if (project.isOpen() == false) {
			String projectName = project.getName();
			String msg = "Project " + projectName + " not opened.";
			throw new ProjectNotOpenedException(msg);
		}
	}

	/**
	 * getProjectContainer
	 * 
	 * @return IContainer
	 */
	public IContainer getProjectContainer() {
		IContainer projContainer = null;

		if (selection != null && selection.isEmpty() == false && selection instanceof IStructuredSelection) {
			IStructuredSelection ssel = (IStructuredSelection) selection;
			if (ssel.size() > 1) {
				return null;
			}
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource) {
				projContainer = ((IResource) obj).getProject();
			}
		}
		return projContainer;
	}

	/**
	 * createControl
	 */
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 3;
		layout.verticalSpacing = 9;
		Label label = new Label(container, SWT.NULL);
		label.setText("File container(&c): ");

		containerText = new Text(container, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		containerText.setLayoutData(gd);
		containerText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				dialogChanged();
			}
		});

		Button button = new Button(container, SWT.PUSH);
		button.setText("...");
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleBrowse();
			}
		});

		label = new Label(container, SWT.NULL);
		label.setText("File name(&m): ");

		fileText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		fileText.setLayoutData(gd);
		fileText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				dialogChanged();
			}
		});
		fileText.setText(idlFileName);
		initialize();
		dialogChanged();
		setControl(container);
	}

	/**
	 * initialize Tests if the current workbench selection is a suitable container
	 * to use.
	 */
	private void initialize() {
		if (selection != null && selection.isEmpty() == false && selection instanceof IStructuredSelection) {
			IStructuredSelection ssel = (IStructuredSelection) selection;
			if (ssel.size() > 1)
				return;
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource) {
				IContainer container;
				if (obj instanceof IContainer)
					container = (IContainer) obj;
				else
					container = ((IResource) obj).getParent();
				containerText.setText(container.getFullPath().toString());
			}
		}
	}

	/**
	 * handleBrowse Uses the standard container selection dialog to choose the new
	 * value for the container field.
	 */
	private void handleBrowse() {
		ContainerSelectionDialog dialog = new ContainerSelectionDialog(getShell(),
		  ResourcesPlugin.getWorkspace().getRoot(), false, "Choose new file container:");
		if (dialog.open() == ContainerSelectionDialog.OK) {
			Object[] result = dialog.getResult();
			if (result.length == 1) {
				containerText.setText(((Path) result[0]).toOSString());
			}
		}
	}

	/**
	 * dialogChanged Ensures that both text fields are set.
	 */
	private void dialogChanged() {
		String container = getContainerName();
		String fileName = getIDLFileName();

		if (container.length() == 0) {
			updateStatus("Enter name of new file container name.");
			return;
		}
		if (fileName.length() == 0) {
			updateStatus("Enter name of new IDL file.");
			return;
		}

		int dotLoc = fileName.lastIndexOf('.');
		if (dotLoc != -1) {
			String ext = fileName.substring(dotLoc + 1);
			if (ext.equalsIgnoreCase("idl") == false) {
				updateStatus("IDL files should be with extention *.idl.");
				return;
			}
		}
		boolean bEnd = fileName.toLowerCase().endsWith(".idl");
		if (bEnd == false) {
			updateStatus("IDL files should be with extention *.idl.");
			return;
		}
		updateStatus(null);
	}

	/**
	 * updateStatus
	 * 
	 * @param message
	 */
	private void updateStatus(String message) {
		setErrorMessage(message);
		setPageComplete(message == null);
	}

	/**
	 * getContainerName
	 * 
	 * @return
	 */
	public String getContainerName() {
		return containerText.getText();
	}

	/**
	 * getIDLFileName
	 * 
	 * @return
	 */
	public String getIDLFileName() {
		return fileText.getText();
	}

	/**
	 * setIDLFileName
	 * 
	 * @param fname
	 */
	public void setIDLFileName(String fname) {
		this.idlFileName = fname;
	}
}
