package org.orbstudio.wizards.server;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.orbstudio.ORBStudioPlugin;
import org.orbstudio.exception.IDLFileNotFoundException;
import org.orbstudio.exception.ProjectNotOpenedException;

/**
 * @stereotype MainController
 */
public class AOMServerWizard extends Wizard implements INewWizard {
	private IWorkbench workbench;
	private IStructuredSelection selection;
	protected String separator = System.getProperties().getProperty("line.separator");

	/**
	 * @clientCardinality 1
	 * @supplierCardinality 0..1
	 */
	private AOMServerWizardPage aomServerWizardPage = null;

	/**
	 * @clientCardinality 1
	 * @supplierCardinality 0..1
	 */
	private AOMServerWizardPage2 aomServerWizardPage2 = null;

	/**
	 * Constructor
	 */
	public AOMServerWizard() {
		super();
	}

	/**
	 * performFinish
	 */
	public boolean performFinish() {
		final IPackageFragmentRoot packageFragmentRoot = aomServerWizardPage.getPackageFragmentRoot();
		final String packageName = aomServerWizardPage.getPackageName();
		final String interfaceName = aomServerWizardPage.getInterface();
		final String implfileName = aomServerWizardPage.getImplFileName();
		final boolean bTie = aomServerWizardPage2.isTie();
		final boolean bServer = aomServerWizardPage2.isServerNeeded();

		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
				try {
					doFinish(packageFragmentRoot, packageName, interfaceName, implfileName, bTie, bServer, monitor);
				}
				catch (CoreException ex) {
					throw new InvocationTargetException(ex);
				}
				finally {
					monitor.done();
				}
			}
		};

		try {
			this.getContainer().run(true, false, op);
		}
		catch (InvocationTargetException ex) {
			Throwable realException = ex.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException.getMessage());
			return false;
		}
		catch (InterruptedException ex) {
			return false;
		}
		return true;
	}

	/**
	 * doFinish Making the mainstream server file based on the active object map.
	 * 
	 * @param packageFragmentRoot
	 * @param packageName
	 * @param interfaceName
	 * @param implfileName
	 * @param monitor
	 * @throws CoreException
	 */
	private void doFinish(IPackageFragmentRoot packageFragmentRoot, String packageName, String interfaceName,
	    String implfileName, boolean bTie, boolean bServer, IProgressMonitor monitor) throws CoreException {
		IPackageFragment packageFragment = null;
		ICompilationUnit compUnitServerImplFile = null;
		@SuppressWarnings("unused")
    ICompilationUnit compUnitServerFile = null;

		monitor.beginTask("Creating CORBA server implementation...", 4);

		/*
		 * Create the package directory.
		 */
		monitor.setTaskName("Creating the package directory...");
		packageFragment = packageFragmentRoot.createPackageFragment(packageName, false, new NullProgressMonitor());
		monitor.worked(1);

		monitor.setTaskName("Creating Java Servant implementation...");
		compUnitServerImplFile = createServerImplFile(packageFragment, packageName, implfileName, interfaceName, bTie);
		monitor.worked(1);

		if (bServer) {
			monitor.setTaskName("Creating Java Server implementation...");
			compUnitServerFile = createServerFile(packageFragment, packageName, implfileName, interfaceName, bTie);
		}
		monitor.worked(1);

		final IFile file = (IFile) compUnitServerImplFile.getResource();
		monitor.setTaskName("Opening Java servant implementation for editing...");
		getShell().getDisplay().asyncExec(new Runnable() {
			public void run() {
				IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
				try {
					IDE.openEditor(page, file, true);
				}
				catch (PartInitException e) {
				}
			}
		});
		monitor.worked(1);
	}

	/**
	 * createServerImplFile Making the servant implementation class.
	 * 
	 * @param packageFragment
	 * @param packageName
	 * @param implfileName
	 * @param iffileName
	 * @param bTie
	 * @return
	 * @throws CoreException
	 */
	private ICompilationUnit createServerImplFile(IPackageFragment packageFragment, String packageName,
	    String implfileName, String iffileName, boolean bTie) throws CoreException {
		ICompilationUnit compUnitFile = null;

		String templatename = null;
		try {
			if (bTie) {
				templatename = ORBStudioPlugin.getDefault().getAOMTIEServerImplFileTemplate().toString();
			}
			else {
				templatename = ORBStudioPlugin.getDefault().getAOMServerImplFileTemplate().toString();
			}
		}
		catch (IOException ex) {
			WizardUtil.getInstance().throwCoreException(ex.getMessage());
		}
		compUnitFile = WizardUtil.getInstance().createUnitFile(packageFragment, templatename, packageName, implfileName,
		  iffileName, implfileName + ".java");
		return compUnitFile;
	}

	/**
	 * createServerFile
	 * 
	 * @param packageFragment
	 * @param packageName
	 * @param implfileName
	 * @param iffileName
	 * @param bTie
	 * @return
	 * @throws CoreException
	 */
	private ICompilationUnit createServerFile(IPackageFragment packageFragment, String packageName, String implfileName,
	    String iffileName, boolean bTie) throws CoreException {
		ICompilationUnit compUnitFile = null;
		String output = null, template = null;

		try {
			if (bTie) {
				template = ORBStudioPlugin.getDefault().getAOMTIEServerFileTemplate().toString();
				output = "Server_AOMtie.java";
			}
			else {
				template = ORBStudioPlugin.getDefault().getAOMServerFileTemplate().toString();
				output = "Server_AOM.java";
			}
		}
		catch (IOException ex) {
			WizardUtil.getInstance().throwCoreException(ex.getMessage());
		}

		compUnitFile = WizardUtil.getInstance().createUnitFile(packageFragment, template, packageName, implfileName,
		  iffileName, output);
		return compUnitFile;
	}

	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.workbench = workbench;
		this.selection = selection;

		setNeedsProgressMonitor(true);
		setWindowTitle("ORB Studio CORBA Wizard");
	}

	/**
	 * addPages
	 */
	public void addPages() {
		super.addPages();

		try {
			aomServerWizardPage = new AOMServerWizardPage(workbench, selection);
		}
		catch (IDLFileNotFoundException ex) {
			MessageDialog.openWarning(this.getShell(), "ORB Wizard", ex.getMessage());
			return;
		}
		catch (ProjectNotOpenedException ex) {
			MessageDialog.openWarning(this.getShell(), "ORB Wizard", ex.getMessage());
			return;
		}
		catch (CoreException ex) {
			MessageDialog.openError(this.getShell(), "ORB Wizard", ex.getMessage());
			return;
		}

		/*
		 * Wizard page (1/2) Ask user to enter the target IDL filename and
		 * interface/package name and the generated file name
		 */
		aomServerWizardPage.setTitle("Server Implementation Wizard 1/2");
		aomServerWizardPage
		    .setDescription("Create a Java server implementation that is based on active object map from the IDL file.");
		URL url = ORBStudioPlugin.getDefault().getIconImage("/icons/server_wiz.gif");
		aomServerWizardPage.setImageDescriptor(ImageDescriptor.createFromURL(url));
		addPage(aomServerWizardPage);

		/*
		 * Wizard page(2/2)
		 */
		aomServerWizardPage2 = new AOMServerWizardPage2(workbench, selection);
		aomServerWizardPage2.setTitle("Server Implementation Wizard 2/2");
		aomServerWizardPage2.setDescription("Choose options for code generation.");
		aomServerWizardPage2.setImageDescriptor(ImageDescriptor.createFromURL(url));
		addPage(aomServerWizardPage2);
	}
}
