package org.orbstudio.wizards.server;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.orbstudio.ORBStudioPlugin;
import org.orbstudio.exception.IDLFileNotFoundException;
import org.orbstudio.exception.ProjectNotOpenedException;

/**
 * @stereotype MainController
 */
public class SLTServerWizard extends Wizard implements INewWizard {
	/**
	 * @stereotype uses
	 */

	/**
	 * @clientCardinality 1
	 * @supplierCardinality 0..1
	 */
	private SLTServerWizardPage sltServerWizardPage = null;

	/**
	 * @clientCardinality 1
	 * @supplierCardinality 0..1
	 */
	private SLTServerWizardPage2 sltServerWizardPage2 = null;

	private IWorkbench workbench;
	private IStructuredSelection selection;
	protected String separator = System.getProperties().getProperty("line.separator");

	/**
	 * SLTServerWizard
	 */
	public SLTServerWizard() {
		super();
	}

	/**
	 * performFinish
	 */
	public boolean performFinish() {
		final IPackageFragmentRoot packageFragmentRoot = sltServerWizardPage.getPackageFragmentRoot();
		final String packageName = sltServerWizardPage.getPackageName();
		final String interfaceName = sltServerWizardPage.getInterface();
		final String implfileName = sltServerWizardPage.getImplFileName();
		final boolean bTie = sltServerWizardPage2.isTie();
		final boolean bServer = sltServerWizardPage2.isServerNeeded();

		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
				try {
					doFinish(packageFragmentRoot, packageName, interfaceName, implfileName, bTie, bServer, monitor);
				}
				catch (CoreException ex) {
					throw new InvocationTargetException(ex);
				}
				finally {
					monitor.done();
				}
			}
		};

		try {
			this.getContainer().run(true, false, op);
		}
		catch (InvocationTargetException ex) {
			Throwable realException = ex.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException.getMessage());
			return false;
		}
		catch (InterruptedException ex) {
			return false;
		}
		return true;
	}

	/**
	 * doFinish
	 * 
	 * @param packageFragmentRoot
	 * @param packageName
	 * @param interfaceName
	 * @param implfileName
	 * @param monitor
	 * @throws CoreException
	 */
	private void doFinish(IPackageFragmentRoot packageFragmentRoot, String packageName, String interfaceName,
	    String implfileName, boolean bTie, boolean bServer, IProgressMonitor monitor) throws CoreException {
		IPackageFragment packageFragment = null;
		ICompilationUnit compUnitServerImplFile = null;
		@SuppressWarnings("unused")
    ICompilationUnit compUnitServantActivatorFile = null;
		@SuppressWarnings("unused")
    ICompilationUnit compUnitServerFile = null;

		monitor.beginTask("Creating CORBA server implementation...", 5);

		monitor.setTaskName("Creating the package directory...");
		packageFragment = packageFragmentRoot.createPackageFragment(packageName, false, new NullProgressMonitor());
		monitor.worked(1);

		monitor.setTaskName("Creating Java Servant implementation...");
		compUnitServerImplFile = createServerImplFile(packageFragment, packageName, implfileName, interfaceName, bTie);
		monitor.worked(1);

		monitor.setTaskName("Creating Java Servant activator implementation...");
		compUnitServerFile = createServantLocatorFile(packageFragment, packageName, implfileName);
		monitor.worked(1);

		if (bServer) {
			monitor.setTaskName("Creating Java Server implementation...");
			compUnitServerFile = createServerFile(packageFragment, packageName, implfileName, interfaceName, bTie);
		}
		monitor.worked(1);

		final IFile file = (IFile) compUnitServerImplFile.getResource();
		monitor.setTaskName("Opening Java servant implementation for editing...");
		getShell().getDisplay().asyncExec(new Runnable() {
			public void run() {
				IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
				try {
					IDE.openEditor(page, file, true);
				}
				catch (PartInitException e) {
				}
			}
		});
		monitor.worked(1);
	}

	/**
	 * createServantLocator
	 * 
	 * @param packageFragment
	 * @param packageName
	 * @param implfileName
	 * @return
	 * @throws CoreException
	 */
	private ICompilationUnit createServantLocatorFile(IPackageFragment packageFragment, String packageName,
	    String implfileName) throws CoreException {
		ICompilationUnit compUnitFile = null;
		String template = null;
		try {
			template = ORBStudioPlugin.getDefault().getSLTServerActivatorTemplate().toString();
		}
		catch (IOException ex) {
			WizardUtil.getInstance().throwCoreException(ex.getMessage());
		}

		compUnitFile = WizardUtil.getInstance().createUnitFile(packageFragment, template, packageName, implfileName, "",
		  "ServantLocatorImpl.java");
		return compUnitFile;
	}

	/**
	 * createServerImplFile
	 * 
	 * @param packageFragment
	 * @param packageName
	 * @param implfileName
	 * @param iffileName
	 * @param bTie
	 * @return
	 * @throws CoreException
	 */
	private ICompilationUnit createServerImplFile(IPackageFragment packageFragment, String packageName,
	    String implfileName, String iffileName, boolean bTie) throws CoreException {
		ICompilationUnit compUnitFile = null;
		String template = null;
		try {
			if (bTie) {
				template = ORBStudioPlugin.getDefault().getSLTTIEServerImplFileTemplate().toString();
			}
			else {
				template = ORBStudioPlugin.getDefault().getSLTServerImplFileTemplate().toString();
			}
		}
		catch (IOException ex) {
			WizardUtil.getInstance().throwCoreException(ex.getMessage());
		}

		compUnitFile = WizardUtil.getInstance().createUnitFile(packageFragment, template, packageName, implfileName,
		  iffileName, implfileName + ".java");
		return compUnitFile;
	}

	/**
	 * createServerFile()
	 * 
	 * @param packageFragment
	 * @param packageName
	 * @param implfileName
	 * @param iffileName
	 * @param bTie
	 * @return
	 * @throws CoreException
	 */
	private ICompilationUnit createServerFile(IPackageFragment packageFragment, String packageName, String implfileName,
	    String iffileName, boolean bTie) throws CoreException {
		ICompilationUnit compUnitFile = null;
		String template = null;
		try {
			template = ORBStudioPlugin.getDefault().getSLTServerFileTemplate().toString();
		}
		catch (IOException ex) {
			WizardUtil.getInstance().throwCoreException(ex.getMessage());
		}

		compUnitFile = WizardUtil.getInstance().createUnitFile(packageFragment, template, packageName, implfileName,
		  iffileName, "Server_ServantLocator.java");
		return compUnitFile;
	}

	/*
	 * ( Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
	 *      org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.workbench = workbench;
		this.selection = selection;

		setNeedsProgressMonitor(true);
		setWindowTitle("ORB Studio CORBA Wizard");
	}

	/*
	 * ( Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 */
	public void addPages() {
		super.addPages();
		try {
			sltServerWizardPage = new SLTServerWizardPage(workbench, selection);
		}
		catch (IDLFileNotFoundException ex) {
			MessageDialog.openWarning(this.getShell(), "ORB Wizard", ex.getMessage());
			return;
		}
		catch (ProjectNotOpenedException ex) {
			MessageDialog.openWarning(this.getShell(), "ORB Wizard", ex.getMessage());
			return;
		}
		catch (CoreException ex) {
			MessageDialog.openError(this.getShell(), "ORB Wizard", ex.getMessage());
			return;
		}
		/*
		 * Wizard page(1/2)
		 */
		sltServerWizardPage.setTitle("Server Implementation Wizard 1/2");
		sltServerWizardPage
		    .setDescription("Create a Java server implementation that is based on servant locator from the IDL file.");
		URL url = ORBStudioPlugin.getDefault().getIconImage("/icons/server_wiz.gif");
		sltServerWizardPage.setImageDescriptor(ImageDescriptor.createFromURL(url));
		addPage(sltServerWizardPage);

		/*
		 * Wizard page(2/2)
		 */
		sltServerWizardPage2 = new SLTServerWizardPage2(workbench, selection);
		sltServerWizardPage2.setTitle("Server Implementation Wizard 2/2");
		sltServerWizardPage2.setDescription("Choose options for code generation.");
		sltServerWizardPage2.setImageDescriptor(ImageDescriptor.createFromURL(url));
		addPage(sltServerWizardPage2);
	}
}
