package org.orbstudio.wizards.server;

import java.util.ArrayList;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.JavaModelException;
import org.orbstudio.ORBStudioPlugin;
import org.orbstudio.common.AppKeys;
import org.orbstudio.common.SystemUtil;
import org.orbstudio.exception.IDLFileNotFoundException;
import org.orbstudio.exception.ProjectNotOpenedException;
import org.orbstudio.popup.actions.factory.IORB;

/**
 * @stereotype Utility
 */
public class WizardUtil {
	/**
	 * Holds singleton instance
	 */
	private static WizardUtil instance;
	protected String separator = System.getProperties().getProperty("line.separator");

	/**
	 * prevents instantiation
	 */
	private WizardUtil() {
	}

	/**
	 * WizardUtil
	 * 
	 * @return the singleton instance
	 */
	static public WizardUtil getInstance() {
		if (instance == null) {
			instance = new WizardUtil();
		}
		return instance;
	}

	/**
	 * createUnitFile()
	 * 
	 * @param packageFragment
	 * @param templateFile
	 * @param packageName
	 * @param implfileName
	 * @param iffileName
	 * @param outputFileName
	 * @return
	 * @throws CoreException
	 */
	public ICompilationUnit createUnitFile(IPackageFragment packageFragment, String templateContent, String packageName,
	    String implfileName, String iffileName, String outputFileName) throws CoreException {
		ICompilationUnit compUnitFile = null;
		//StringBuffer sb = new StringBuffer();
		IORB currentORB = ORBStudioPlugin.getDefault().getCurrnetORB();

		String code = templateContent;
		/*
		 * Build Java code by replacing some parameters.
		 */
		code = SystemUtil.strReplace(code, "%%%SERVERIMPLCLASSNAME%%%", implfileName);
		code = SystemUtil.strReplace(code, "%%%SERVERINTERFACE%%%", iffileName);
		code = SystemUtil.strReplace(code, "%%%SERVERPROPERTIES%%%", currentORB.getORBPropertiesCode().toString());
		/*
		 * Note. Java IDL seems not to support 'Persistence policy' so make policy
		 * parts commented out.
		 */
		if (currentORB.getORBname().equals(AppKeys.ORB_JAVAIDL)) {
			code = SystemUtil.strReplace(code, "%%%JAVAIDL_COMMENTOUT%%%", "// ");
		}
		else {
			code = SystemUtil.strReplace(code, "%%%JAVAIDL_COMMENTOUT%%%", "");
		}

		try {
			compUnitFile = packageFragment.createCompilationUnit(outputFileName, code, true, new NullProgressMonitor());

			/**
			 * Add package declare.
			 */
			if (!packageFragment.isDefaultPackage()) {
				compUnitFile.createPackageDeclaration(packageName, new NullProgressMonitor());
			}
		}
		catch (JavaModelException ex) {
			throwCoreException(ex.getMessage());
		}
		return compUnitFile;
	}

	/**
	 * throwCoreException
	 * 
	 * @param message
	 * @throws CoreException
	 */
	public void throwCoreException(String message) throws CoreException {
		IStatus status = new Status(IStatus.ERROR, "org.orbstudio", IStatus.OK, message, null);
		throw new CoreException(status);
	}

	/**
	 * getIDLfiles
	 * 
	 * @return
	 * @throws ProjectNotOpenedException
	 * @throws CoreException
	 * @throws IDLFileNotFoundException
	 */
	public IFile[] getIDLfiles(IProject project) throws ProjectNotOpenedException, CoreException,
	    IDLFileNotFoundException {
		ArrayList<IFile> idlFiles = new ArrayList<IFile>();

		/*
		 * Check if the project is opened already.
		 */
		if (project.isOpen() == false) {
			String projectName = project.getName();
			String msg = "Project " + projectName + " not opened.";
			throw new ProjectNotOpenedException(msg);
		}

		IResource[] members;
		members = project.members();
		for (int i = 0; i < members.length; i++) {
			countIDLFile(members[i], idlFiles);
		}

		/**
		 * In case the project is not yet opened...
		 */
		if (idlFiles.size() == 0) {
			String projectName = project.getName();
			String msg = "No IDL files found in the project, " + projectName;
			throw new IDLFileNotFoundException(msg);
		}
		IFile[] list = (IFile[]) idlFiles.toArray(new IFile[idlFiles.size()]);
		return list;
	}

	/**
	 * countIDLFile
	 * 
	 * @param resource
	 * @param idlFiles
	 * @throws CoreException
	 */
	private void countIDLFile(IResource resource, ArrayList<IFile> idlFiles) throws CoreException {
		switch (resource.getType()) {
			case IResource.FOLDER:
				IFolder folder = (IFolder) resource;
				IResource[] resources = folder.members();
				for (int i = 0; i < resources.length; i++) {
					countIDLFile(resources[i], idlFiles);
				}
				break;

			case IResource.FILE:
				IFile file = (IFile) resource;
				String ext = file.getFileExtension();
				if (ext.equalsIgnoreCase("idl")) {
					idlFiles.add(file);
				}
				break;
		}
	}

	/**
	 * isProject
	 * 
	 * @param element
	 * @return
	 */
	public boolean isProject(Object element) {
		return (element instanceof IJavaProject) || (element instanceof IProject);
	}
}
